<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class UrlsController extends Controller{

    private function _getUrlsAdmin(){
        return DB::table('urls')
            ->select(DB::raw('
                urls.id, urls.url,
                (select count(url) from links 
                    where links.url=urls.url 
                ) as link_count
            '))->get()
        ;
    }
    private function _getLinksAdmin($url){
        return DB::table('links')
            ->select(DB::raw('
                id, url, link
            '))->where("url", $url)->get()
        ;
    }

    private function _getUrlLinkCount($url){
        $data=DB::table('links')->select(DB::raw("count(id) as count"))->where('url', $url)->first();

        if($data){
            return intval($data->count);
        }

        return 0;
    }

    private function _urlExists($url){
        $data=DB::table('urls')->select("id")->where('url', $url)->first();

        if($data){
            return true;
        }else{
            return false;
        }
    }

    private function _linkUrlExists($url, $link){
        $data=DB::table('links')->select("id")->where('url', $url)->where('link', $link)->first();

        if($data){
            return true;
        }else{
            return false;
        }
    }

    private function _getLinkByIdUrl($url, $link_id){
        $data=DB::table('links')->select("id", "url", "link")->where('url', $url)->where('id', $link_id)->first();

        if($data){
            return $data;
        }else{
            return false;
        }
    }

    private function _deleteLink($link_id){
        DB::table("links")->where('id', $link_id)->delete(); 
    }

    private function _addLink($url, $link){
        DB::table('links')->insertGetId([
            'url' => $url,
            'link' => $link,
        ]);
    }

    private function _addUrl($url, $link){
        $zero=0;
        DB::table('urls')->insertGetId([
            'url' => $url,
            'last_key' => $zero,
        ]);

        UrlsController::_addLink($url, $link);
    }





    public function move($url){
        $urlData=DB::table('urls')->select("last_key")->where('url', $url)->first();
        if($urlData){
            $key=intval($urlData->last_key);

            $links=DB::table('links')->select("link")->where('url', $url)->get();
            if(!isset($links[$key])){
                $key=0;
            }


            DB::table('urls')->where('url', $url)->update([
                "last_key"=>$key+1
            ]);

            return redirect($links[$key]->link);

        }


        return redirect("https://google.com/");
    }

    public function index(){
        $urlDataList=UrlsController::_getUrlsAdmin();
        return view("admin_urls", ["urlDataList"=>$urlDataList]);
    }

    public function links($url){
        $linksDataList=UrlsController::_getLinksAdmin($url);
        return view("admin_links", [
            "linksDataList"=>$linksDataList,
            "url"=>$url
        ]);
    }

    public function delete_link($url, $link_id){
        $data=UrlsController::_getLinkByIdUrl($url, $link_id);

        $messages=[];
        $err=false;

        if(!$data){
            $err=true;
    
            array_push($messages, 
                [
                    "isErr"=>true,
                    "msg"=>"Some error occured"
                ]
            );
        }

        if(!$err){
            $count=UrlsController::_getUrlLinkCount($url);
            if($count < 2){
                $err=true;
    
                array_push($messages, 
                    [
                        "isErr"=>true,
                        "msg"=>"Number of liks can not be zero"
                    ]
                );
            }
        }


        if(!$err){
            UrlsController::_deleteLink($link_id);

            array_push($messages, 
                [
                    "isErr"=>false,
                    "msg"=>"Link deleted"
                ]
            );
            
        }

        return redirect("/admin/$url/links")->with(session()->flash('messages', $messages));
    }

    public function delete_url($url){
        $messages=[];
        if(UrlsController::_urlExists($url)){
            DB::table("links")->where('url', $url)->delete(); 
            DB::table("urls")->where('url', $url)->delete(); 

            array_push($messages, 
                [
                    "isErr"=>false,
                    "msg"=>"URL deleted"
                ]
            );

        }

        return redirect("/admin/urls")->with(session()->flash('messages', $messages));
    }

    public function addUrl(Request $request){
        $url_n=$request->url_n;
        $link=$request->link;

        $messages=[];
        

        $err=false;

        if(!strlen($link)){
            $err=true;
    
            array_push($messages, 
                [
                    "isErr"=>true,
                    "msg"=>"Please fill in the link"
                ]
            );
        }
        if(!strlen($url_n)){
            $err=true;
    
            array_push($messages, 
                [
                    "isErr"=>true,
                    "msg"=>"Please fill in URL"
                ]
            );
        }


        if(!$err){
            if(UrlsController::_urlExists($url_n)){
                $err=true;
    
                array_push($messages, 
                    [
                        "isErr"=>true,
                        "msg"=>"URL already exists"
                    ]
                );
            }
        }


        if(!$err){
            UrlsController::_addUrl($url_n, $link);
            array_push($messages, 
                [
                    "isErr"=>false,
                    "msg"=>"URL added"
                ]
            );
    
        }


        return redirect($request->getRequestUri())->with(session()->flash('messages', $messages));
    }

    public function addLink(Request $request, $url){
        $link=$request->link;

        $messages=[];
        

        $err=false;

        if(!strlen($link)){
            $err=true;
    
            array_push($messages, 
                [
                    "isErr"=>true,
                    "msg"=>"Please fill in the link"
                ]
            );
        }


        if(!$err){
            if(UrlsController::_linkUrlExists($url, $link)){
                $err=true;
    
                array_push($messages, 
                    [
                        "isErr"=>true,
                        "msg"=>"Link already exist in URL"
                    ]
                );
            }
        }


        if(!$err){
            UrlsController::_addLink($url, $link);
            array_push($messages, 
                [
                    "isErr"=>false,
                    "msg"=>"Link added"
                ]
            );
    
        }


        return redirect($request->getRequestUri())->with(session()->flash('messages', $messages));
    }
}
